package timestamp;
use strict;
require 'sys/syscall.ph';

BEGIN{
    require Exporter;
    use vars qw(@ISA @EXPORT);
    @ISA = qw(Exporter);
    @EXPORT = qw(&nowts &nowtval &ntptots &tstotval &tvaltots 
		 &tstofloat &floattots &diffmod &sdifftofloat);
}

sub pushbit {
    my ( $v,$b ) = @_;
    return (( $v << 1 ) | $b);
}

sub popbit {
    my ( $v_ptr ) = @_;
    my $res = $$v_ptr & 0x1;
    $$v_ptr = $$v_ptr >> 1;
    return $res;
}

sub nowts {
    my ( $scale ) = @_;
    my $time = pack("LL", ());
    syscall(&SYS_gettimeofday, $time, 0) != -1
	|| die "gettimeofday: $!";
    return tvaltots($scale, $time);
}

sub nowtval {
    my $time = pack("LL", ());
    syscall(&SYS_gettimeofday, $time, 0) != -1
	|| die "gettimeofday: $!";
    return $time;
}


sub ntptots {
    my ( $scale, $ntp ) = @_;
}

sub tstotval {
    my ( $scale, $timestamp ) = @_;
    my @t;

    my $decimal_size=$scale+10;

    $t[0] = $timestamp >> $decimal_size;
    $t[1] = 0.0;
    my $decimal_part = $timestamp & ( ~ ( 0xFFFFFFFF << $decimal_size ));
    
    for ( ; $decimal_size > 0; $decimal_size--) {
	$t[1] = ( $t[1] + popbit(\$decimal_part) ) / 2;
    }
    $t[1] *= 1000000;
    if ( ( $t[1] - int($t[1]) ) > 0.5 ) {
	$t[1]=int($t[1])+1;
    } else {
	$t[1]=int($t[1]);
    }
    return pack("LL",@t);
}

sub tstofloat {
    my ( $scale, $timestamp ) = @_;
    my @t;

    my $decimal_size=$scale+10;

    $t[0] = $timestamp >> $decimal_size;
    $t[1] = 0.0;
    my $decimal_part = 
	$timestamp & ( ~ ( 0xFFFFFFFF << $decimal_size ));
    
    for ( ; $decimal_size > 0; $decimal_size--) {
	$t[1] = ( $t[1] + popbit(\$decimal_part) ) / 2;
    }
    return ($t[0]+$t[1]);
}

sub tvaltots {
    my ( $scale, $tval ) = @_;
    my @t=unpack("LL",$tval);
    (( $scale >= 0x0 ) && ( $scale <= 0xF )) || die "Time scale out of bound";

    my $decimal_size=$scale+10;
    my $integer_size=32-$decimal_size;
    my $timestamp;

    $timestamp = $t[0] & ( ~ ( 0xFFFFFFFF << $integer_size ));

    my $usec=$t[1];
    for (; $decimal_size>0; $decimal_size--) {
	$usec *= 2;
	if ( $usec >= 1000000 ) {
	    $timestamp=pushbit($timestamp,0x1);
	    $usec -= 1000000;
	} else {
	    $timestamp=pushbit($timestamp,0x0);
	}
    }
    return $timestamp;
}

sub floattots {
    my ( $scale, $secs ) = @_;
    my @t=( int($secs), int(($secs-int($secs))*1000000) );
    (( $scale >= 0x0 ) && ( $scale <= 0xF )) || die "Time scale out of bound";

    my $decimal_size=$scale+10;
    my $integer_size=32-$decimal_size;
    my $timestamp;

    $timestamp = $t[0] & ( ~ ( 0xFFFFFFFF << $integer_size ));

    my $usec=$t[1];
    for (; $decimal_size>0; $decimal_size--) {
	$usec *= 2;
	if ( $usec >= 1000000 ) {
	    $timestamp=pushbit($timestamp,0x1);
	    $usec -= 1000000;
	} else {
	    $timestamp=pushbit($timestamp,0x0);
	}
    }
    return $timestamp;
}

# Questa restituisce la differenza (con il segno giusto), a partire
# dai timestamps senza segno se  meno di 2^31. Esperimenti in
# provamod4.pl.  

sub diffmod { 
    use integer; 
    my ($a,$b) = @_; 
    return $a-$b; 
}

# Questa converte una differenza con segno scalata tra timestamps 
# in un valore temporale in secondi in virgola mobile (rappresentazione 
# Perl)

sub sdifftofloat {
my ($d,$s) = @_;
    return ($d/(0x1<<($s+10)));
}


1;







