package Session;
use strict;
use IO::Socket;
use packet;
use timestamp;
use spqrcrypt;

BEGIN{
    require Exporter;
    use vars qw(@ISA @EXPORT $VERSION);
    $VERSION=0;
    @ISA = qw(Exporter);
    @EXPORT = qw(&new &open &ping );
}

sub new {
    shift;
    my %arg;
    my $configfile=shift;
    open CONFIG, $configfile 
	or die "$configfile: $!";
    while (<CONFIG>) {
	my ($name, $value)= split;
	$arg{$name}=$value;
    }
    close CONFIG;
#    my %arg = @_;
    my $self = \%arg;
    my $target_port=22000;
    $self->{socket_out} = 
      IO::Socket::INET->new(
			    Proto    => "UDP",
			    PeerAddr => $self->{targetIP},
			    PeerPort => $target_port
			    )
	or die "socket out: $!";
    $self->{socket_in} = 
      IO::Socket::INET->new(
			    Proto    => "UDP",
			    LocalAddr => $self->{probeIP},
			    LocalPort => $self->{port}
			    )
	or die "socket in: $!";
    bless $self;
    return $self;
}

=pod
IN = session hash
DESCRIPTION = Takes a reference to a session, sends a open 
message based on it, and waits for the reply. 
Returns upon success, otherwise dies.
=cut
sub open {
    my $self = shift;
# Build composite fields
    my $VT = ( $VERSION << 4) | $packet::messageType{open};
    my $period = 
	( $self->{scale} << 28 ) | 
	(floattots($self->{scale}, $self->{period})  & 0xFFFFFFF );
# Builds OPEN message
    my $message = pack ("CCnNSn",
		       $VT, 
		       0x0, 
		       $self->{length}, 
		       $period, 
		       0x0, 
		       $self->{port});
# Send OPEN message
    defined( send ($self->{socket_out}, $message, 0) )
	|| die "send OPEN: $!";
    my $rin = '';
# Wait 10 secs for ACCEPT
    vec ($rin, fileno($self->{socket_in}), 1) = 1;
    select($rin, undef, undef, 10)
	|| die "Open message timed out!\n";
# Receive ACCEPT
    my $reply='';
    defined( recv ($self->{socket_in}, $reply, $packet::maxbuffer, 0) )
	|| die "send OPEN: $!";
# Unpack ACCEPT
    my ( $targetVT, $targetlength, $targetperiod, $targetsignature ) = 
	unpack("CxnNA4",$reply);
# Process ACCEPT
    ( ( $targetVT >> 4 ) == $VERSION ) || 
	die "Version mismatch\n";
    ( ( $targetVT & 0xF ) == $packet::messageType{reply}) || 
	die "Unmatched open\n";
    ( $self->{length} > $targetlength ) &&
	( $self->{length} = $targetlength );
# PERICOLOSO (ci sarebbe da fare qualche test di accettabilita')
    $self->{period} = 
	tstofloat($self->{scale}, $targetperiod & 0xFFFFFFF);
    $self->{scale} = $targetperiod >> 28;
    $self->{signature} = $targetsignature;
}

=pod
IN = session hash
DESCRIPTION = Takes a reference to a session, sends a forward message 
message based on its description, and waits for the backward message.
Returns, upon success, an array containing the four timestamps recorded 
in the roudtrip.
=cut
sub ping {
    my $self = shift;
# Build composite fields
    my $VT = ( $VERSION << 4) | $packet::messageType{forward};
    my $key=spqrcrypt::newhex7;
    my $check=spqrcrypt::checkwords($key,$self->{signature});
#    print "checkwords=$check->[0] $check->[1] $key\n";
    my $timestamp = ( $self->{scale} << 28 ) | 
#	( nowts($self->{scale}) & 0xFFFFFFF );
	hex $key;
# Builds random message
    my @sentdata=();
    for (my $i=0; $i<$self->{length}-16; $i++) { 
	push @sentdata, int(rand(256));
    }
    my $message = pack ("CCnNSnA4C*",
			$VT, 
			0x0, 
			$self->{length}, 
			$timestamp, 
			0x0, 
			$self->{port},
			$check->[0],
			@sentdata
			);
# =====================================
# Send forward message
    my $t0 = tssend ($self->{socket_out}, $message, $self->{scale})
	|| die "send OPEN: $!";
    my $rin = '';
# Wait 10 secs for a reply
    vec ($rin, fileno($self->{socket_in}), 1) = 1;
    select($rin, undef, undef, 10)
	|| die "Forward message timed out!\n";
# Receive the reply
    my $backward='';
    my ( $targetaddr, $t3tval) =
	tsrecv ($self->{socket_in}, $backward);
# =====================================
    my $t3=tvaltots($self->{scale},$t3tval);
# Unpack the reply
    my ( $targetVT, $targetlength, $authentication, 
	 $t1, $t2, @recvdata ) = unpack("CxnA4NNC*",$backward);
# Process the reply
    if ( ( $targetVT >> 4 ) != $VERSION ) { 
	die "Version mismatch\n";
    }
    if ( ( $targetVT & 0xF ) != $packet::messageType{backward}) { 
	die "Unmatched forward\n";
    }
    if ( $authentication ne $check->[1] ) { 
	die "Authentication failed\n";
    }
    for (my $i=0; $i<$self->{length}-16; $i++) {
#     print "$sentdata[$i], $recvdata[$i]\n";
	if ( $sentdata[$i] != $recvdata[$i]) { 
	    warn "Data error\n";
	    return;
	}
    }
    
    return ( $t0, $t1, $t2, $t3 );
}

1;















